# This is a -*- Tcl -*- library script and belongs to the
# `Jumble Library for Tcl and Friends'.
#
# misc.tcl --- Miscellaneous Functions
#
# Copyright (C) 1996, 1997 Stefan Hornburg
#
# Author: Stefan Hornburg <racke@gundel.han.de>
# Maintainer: Stefan Hornburg <racke@gundel.han.de>
# Version: 0.0a3
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
# 
# This file is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

juf_dbg_add_flags juf_flags_misc_name

# ---------------------------------------
# juf_sortbyval ARRAY
#
# Returns list of element names of ARRAY,
# sorted according to the element values
# ---------------------------------------

proc juf_sortbyval {array} {
	upvar $array parr
	#
	# build auxiliary array
	#
	foreach key [array names parr] {
		lappend v_hash($parr($key)) $key
	}
	puts stdout [array names v_hash]
	set v_list ""
	#
	# build result list
	#
	foreach key [lsort [array names v_hash]] {
		set v_keys $v_hash($key)
		foreach v_key $v_keys {
			lappend v_list $key
		}
	}
	return $v_list
}

proc juf_f2arr {file name} {
	upvar $name parr
	set v_chl [open $file]

	if {! [juf_file_slurp $file v_buf] } {
		return 0
	}
	
	#
	# split into single lines
	#
	set v_list [split $v_buf "\\\n"]
	#
	foreach v_line $v_list {
		set v_chunks [split $v_line]
		set parr([lindex $v_chunks 0]) [lindex $v_chunks end]
	}
}

# -----------------------------------------------------
# juf_map FUNCTION LIST
#
# Applies FUNCTION to each element of LIST in turn, and
# returns a list of the results.
# -----------------------------------------------------

proc juf_map {function list {name ""}} {
	set result ""
	foreach el $list {
		if {$name != ""} {
			set $name $el
			lappend result [eval $function]
		} else {
			lappend result [eval $function {$el}]
		}
	}
	set result
}

# ---------------------------------------------------------
# juf_misc_scriptname - returns name of current Tcl script
# ---------------------------------------------------------

proc juf_misc_scriptname {} {
	file rootname [file tail [info script]]
}

# --------------------------------------------------------
# juf_misc_shell - prompts for user input and evaluates it
#                  in the current interpreter until user
#				   exits the shell
#
# in:	prompt	user prompt (default "% ")
# --------------------------------------------------------

proc juf_misc_shell {{prompt "% "}} {
	puts -nonewline stdout $prompt
	flush stdout

	while {[gets stdin input] != -1} {
		catch "$input" result
		puts stdout $result
		puts -nonewline stdout $prompt
		flush stdout
	}
}

# ---------------------------------------------------------------
# juf_log_versioninfo - log version information on stdout
#
# If a script file is currently being evaluated, this function
# looks for the variables `version', `spec' and `pkginfo' with
# a prefix dependent on the script file name in the global
# context and creates version information from the values of
# this variables. At least the variable `version' must exist.
# E.g., if you running the script `~/foo', the variables 
# `foo_version' (script version), `foo_spec' (script description)
# and `foo_pkginfo' (info about package the script belongs to)
# will be looked up.
# ---------------------------------------------------------------

proc juf_log_versioninfo { {myname ""} {chlid stdout} } {
	set info 0

	if {$myname == ""} {
		set myname [file rootname [file tail [info script]]]
	}
	set outbuf ""

	juf_dbg_out juf_flags_misc_name "Script name is $myname"

	if [uplevel #0 info exists [set myname]_spec] {
		upvar #0 [set myname]_spec buf
		set outbuf "This is $buf version "
	}
	if [uplevel #0 info exists [set myname]_version] {
		upvar #0 [set myname]_version buf
		append outbuf $buf
		set info 1
	}
	if [uplevel #0 info exists [set myname]_pkginfo] {
		upvar #0 [set myname]_pkginfo buf
		append outbuf ", from $buf"
	}
	if {$info} {
		puts $chlid "$outbuf."
	}
}
